// Copyright (c) 2021 The Chromium OS Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
#include "base/logging.h"

#include "arm_spe_parser.h"

namespace quipper {
inline bool ArmSpeParser::IsTargetEvent(const ArmSpeParsedPacket& rawPacket) const
{
  if (rawPacket.packetType & BIT(ArmSpeRawType::EV_RETIRED)
    || rawPacket.packetType & BIT(ArmSpeRawType::EV_LLC_REFILL)
    || rawPacket.packetType & BIT(ArmSpeRawType::EV_L1D_REFILL)) {
      return true;
  }
  return false;
}

bool ArmSpeParser::ParserRawSpeData()
{
  if (!m_armSpePacketDecoder.Init(m_rawTraceData)) {
    m_armSpePacketDecoder.HandleError();
    return false;
  }
  while (m_armSpePacketDecoder.IsUnsolvedPacketExist()) {
    if (ArmSpeParsedPacket packet; m_armSpePacketDecoder.SolveOnePacket(packet)) {
      if (IsTargetEvent(packet)) {
        SavePacket(packet);
      }
    } else {
      m_armSpePacketDecoder.HandleError();
      return false;
    }
  }
  return true;
}

const std::vector<std::string> ArmSpeParser::GetArmSpeEventNameString(ArmSpeEventType type) const
{
  std::vector<std::string> ret;
  for (size_t i = 0; i < sizeof(ArmSpeEventType) * 8; i++) {
    if (type & BIT(i)) {
      switch (i) {
        case ArmSpeRawType::EV_RETIRED:
          ret.push_back("ARM_SPE_INST_RETIRED");
          break;
        case ArmSpeRawType::EV_LLC_REFILL:
          ret.push_back("ARM_SPE_LLC_MISS");
          break;
        case ArmSpeRawType::EV_L1D_REFILL:
          ret.push_back("ARM_SPE_L1D_MISS");
          break;
        case ArmSpeRawType::EV_TLB_REFILL:
        case ArmSpeRawType::EV_MISPRED:
        case ArmSpeRawType::EV_REMOTE_ACCESS:
        case ArmSpeRawType::EV_NOT_TAKEN:
        case ArmSpeRawType::EV_EXCEPTION_GEN:
        case ArmSpeRawType::EV_L1D_ACCESS:
        case ArmSpeRawType::EV_LLC_ACCESS:
        case ArmSpeRawType::EV_TLB_ACCESS:
          /* Useless for AutoFDO, just skip it. */
          break;
        default:
          LOG(FATAL) << "ARM SPE: ArmSpeEventType not support this type: "
                     << static_cast<int>(type);
      }
    }
  }
  return ret;
}

} // namespace quipper